package jnyqide;

import java.io.*;
import javax.swing.*;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author unascribed
 * @version 1.0
 */

public class NyquistThread extends Thread {

    Process myProcess;
    InputStream stdout;
    InputStreamReader isrStdout;
    BufferedReader brStdout;
    OutputStream stdin;
    byte b[];
//  JTextPane textIn;
    JTextArea textOut;
    Runnable update;
    String plotFilename;
    int plotNow;
    PlotFrame plotFrame;
    MainFrame mainFrame; // used to find current directory

    public NyquistThread() {
    }

/*  public void setInputArea( JTextPane in )
  {
    textIn = in;
  }*/

    public void start(JTextArea out, Runnable u, 
              PlotFrame plot, MainFrame main) {
        textOut = out;
        update = u;
        plotFrame = plot;
        mainFrame = main;

        try {
            // Get the current working directory where we can find lib and runtime
            // get the "." file present in all directories
            //java.io.File f = new java.io.File(".");
            // get the absolute path to the "." file
            //String cwd2 = f.getAbsolutePath();
            // return the absolute path minus the "."
            //cwd2 = cwd2.substring(0, cwd2.length() - 2);
            // the code above works, but the following is simpler:
            String cwd = System.getProperty("user.dir");
            String path = "XLISPPATH=";
            path = path.concat(cwd).concat("/lib/:").concat(cwd).concat("/runtime/");
            // this handles the default lib and runtime directories, but what
            // if the user has other directories for the path? Java does not 
            // support getenv() and other system-dependent environment access, so
            // when using the IDE, we will read the path from a file named xlisppath:
            String userpath;
            try {
                BufferedReader br = new BufferedReader(new FileReader("xlisppath"));
                userpath = br.readLine();
                br.close();
            } catch (Exception e) {
                userpath = "";
            }
            path = path.concat(":").concat(userpath);
            System.out.print("XLISPPATH will be "); System.out.println(path);
            String[] envp = {path, "USER=nyquist"};
            try {
                myProcess = Runtime.getRuntime().exec( "./ny", envp );
            } catch (Exception e3) {
                System.out.println("no ./ny found, trying ./nyquist");
                myProcess = Runtime.getRuntime().exec("./nyquist", envp);
                // if this one fails, we'll take the exception below
            }
            System.out.print("myProcess: ");
            System.out.println(myProcess);
            stdout = myProcess.getInputStream();
            isrStdout = new InputStreamReader(stdout);
            brStdout = new BufferedReader(isrStdout);
            stdin = myProcess.getOutputStream();
            stdin.write(6); // control-f turns off console echo for Mac and Linux
            stdin.flush();
        }
        catch( Exception e2 ) { System.out.println(e2); }

        b = new byte[1000];
        if (brStdout != null) {
            super.start();
        } else {
            textOut.append("Nyquist (ny or nyquist) not found or could not be started.");
        }
    }

    public void run() {
        StringBuffer buffer = new StringBuffer(256);
        int buffer_index = 0;
        boolean nyquist_is_running = true;
        // this is tricky -- we want to accumulate lines to test
        // for plot commands, but we want to append chars to 
        // textOut even if there is no end-of-line.
        // 
        // We're going to flush the buffer after every line.
        // That will be expensive if lines are very short, but
        // that's pretty rare. So we'll loop until we either
        // accumulate a line or there are no characters ready.
        // To avoid re-appending characters to textOut, use
        // buffer_index to remember how many characters were
        // output from the current line.
        //
        // Algorithm:
        //   accumulate chars while input is ready and newline not found
        //   if we have a newline, test for plot command
        //   append characters to text box
        //   if we have a newline, reset the buffer

        String line = null;
        while (nyquist_is_running) {
            try {
                // block until you read at least one character
                //System.out.println("block on char");
                int ic = brStdout.read();
                //System.out.println("got char");
                if (ic != -1)  buffer.append((char) ic);
                while (brStdout.ready() && ic != '\n' && ic != -1) {
                    //System.out.println("block on char in while");
                    ic = brStdout.read();
                    //System.out.println("got it");
                    if (ic != -1) buffer.append((char) ic);
                }
                if ((char) ic == '\n') {
                    line = new String(buffer);
                    testForPlot(line);
                } else if (ic == -1) {
                    buffer.append("The Nyquist process has terminated. Please save your files and exit.\n");
                    nyquist_is_running = false;
                }
		// System.out.print("text from Nyquist: ");
                // System.out.println(buffer.substring(buffer_index));
                textOut.append(buffer.substring(buffer_index));
                buffer_index = buffer.length();
                if (line != null) { // end of line, reset the buffer
                    line = null;
                    buffer.setLength(0);
                    buffer_index = 0;
                    // limit number of lines in the textOut box
                    int lines = textOut.getLineCount();
                    if (lines > 5000) {
                        // take out about 100 lines at a time:
                        textOut.replaceRange("", 0, 
                            textOut.getLineStartOffset(lines - 4900));
                    }
                }
                // cause the scroll to go to the end of the new text
                SwingUtilities.invokeLater(update);
            } catch (Throwable t) {
                t.printStackTrace();
            }
    /*      
            try {

            while( stdout.available() > 0 ) {
                int howMany = stdout.read(b, 0, stdout.available());
                String output = new String(b, 0, howMany);
                testForPlot(output);
                textOut.append(output);

                // cause the scroll to go to the end of new text
                // System.out.println(output);
                SwingUtilities.invokeLater(update);
            }
        
            try {
                Thread.sleep((int)( Math.random()*300 ));
            } catch ( InterruptedException e ) {
                e.printStackTrace();
            }
            }
            catch( Exception e ) { System.out.println(e); }
     */
        }
    }

    public void testForPlot(String output) {
        int iNameStart;
        int iNameStop;
        //System.out.print("plot's text output:");
        //System.out.println(output);
        if (plotNow == 0) {
            iNameStart = output.indexOf("s-plot: writing ");
            if (iNameStart >= 0) {
            iNameStop = output.indexOf(" ...");
            plotFilename = output.substring(iNameStart + 16, iNameStop);
            if (!mainFrame.currentDir.equals("")) {
                plotFilename = mainFrame.currentDir + plotFilename;
            }
            // System.out.println("file:" + plotFilename + ":");
            plotNow = 1;
            }
        }
        // this is not ...else if... because output can have many lines
            if (plotNow == 1) {
            iNameStart = output.indexOf(" points from");
            // now, the file is written and closed so we can open it
            if (iNameStart >= 0) {
            // System.out.println("Calling NyqPlot.plot");
            NyqPlot.plot(plotFilename, plotFrame);
            plotNow = 0;
            }
        }
    }

    public OutputStream getStdin() {
        return stdin;
    }

    public void sendInput( String s )
    {
        // stdin = myProcess.getOutputStream();
        // System.out.println(s);
        //    if( s.equals("(exit)\n") )
        //     System.exit(0);
        //   s.concat("\n"); needed???
        try {
            textOut.append(s);
            stdin.write( s.getBytes() );
            stdin.flush();
        }
            catch( Exception e ) { System.out.println(e); 
		}
    }
}









